@@ -12,7 +12,7 @@ use AnyEvent::WebSocket::Connection;
 use PerlX::Maybe qw( maybe provided );
 
 # ABSTRACT: WebSocket client for AnyEvent
-our $VERSION = '0.33_01'; # TRIAL VERSION
+our $VERSION = '0.34_01'; # TRIAL VERSION
 
 
 has timeout => (
@@ -115,7 +115,7 @@ AnyEvent::WebSocket::Client - WebSocket client for AnyEvent
 
 =head1 VERSION
 
-version 0.33_01
+version 0.34_01
 
 =head1 SYNOPSIS
 
@@ -298,6 +298,8 @@ L<RFC 6455 The WebSocket Protocol|http://tools.ietf.org/html/rfc6455>
 
 =back
 
+=for stopwords Joaquín José
+
 =head1 AUTHOR
 
 Author: Graham Ollis E<lt>plicease@cpan.orgE<gt>
@@ -306,6 +308,8 @@ Contributors:
 
 Toshio Ito
 
+José Joaquín Atria
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Graham Ollis.
