@@ -6,7 +6,7 @@ use Moo;
 use Encode ();
 
 # ABSTRACT: WebSocket message for AnyEvent
-our $VERSION = '0.33_01'; # TRIAL VERSION
+our $VERSION = '0.34_01'; # TRIAL VERSION
 
 
 has body => ( is => 'ro', required => 1 );
@@ -39,7 +39,7 @@ AnyEvent::WebSocket::Message - WebSocket message for AnyEvent
 
 =head1 VERSION
 
-version 0.33_01
+version 0.34_01
 
 =head1 SYNOPSIS
 
@@ -134,6 +134,8 @@ L<RFC 6455 The WebSocket Protocol|http://tools.ietf.org/html/rfc6455>
 
 =back
 
+=for stopwords Joaquín José
+
 =head1 AUTHOR
 
 Author: Graham Ollis E<lt>plicease@cpan.orgE<gt>
@@ -142,6 +144,8 @@ Contributors:
 
 Toshio Ito
 
+José Joaquín Atria
+
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2013 by Graham Ollis.
